@@ -188,7 +188,7 @@ func (m *jsMinifier) minifyStmt(i js.IStmt) {
 	case *js.LabelledStmt:
 		m.write(stmt.Label)
 		m.write(colonBytes)
-		m.minifyStmt(stmt.Value)
+		m.minifyStmtOrBlock(stmt.Value, defaultBlock)
 	case *js.BranchStmt:
 		m.write(stmt.Type.Bytes())
 		if stmt.Label != nil {
@@ -306,6 +306,10 @@ func (m *jsMinifier) minifyStmt(i js.IStmt) {
 		if stmt.Catch != nil {
 			m.write(catchBytes)
 			stmt.Catch.List = optimizeStmtList(stmt.Catch.List, defaultBlock)
+			if v, ok := stmt.Binding.(*js.Var); ok && v.Uses == 1 {
+				stmt.Catch.Scope.Declared = stmt.Catch.Scope.Declared[1:]
+				stmt.Binding = nil
+			}
 			m.renamer.renameScope(stmt.Catch.Scope)
 			if stmt.Binding != nil {
 				m.write(openParenBytes)
@@ -358,7 +362,7 @@ func (m *jsMinifier) minifyStmt(i js.IStmt) {
 		if stmt.Default != nil || len(stmt.List) != 0 {
 			m.write(fromBytes)
 		}
-		m.write(minifyString(stmt.Module))
+		m.write(minifyString(stmt.Module, false))
 		m.requireSemicolon()
 	case *js.ExportStmt:
 		m.write(exportBytes)
@@ -395,11 +399,13 @@ func (m *jsMinifier) minifyStmt(i js.IStmt) {
 			}
 			if stmt.Module != nil {
 				m.write(fromBytes)
-				m.write(minifyString(stmt.Module))
+				m.write(minifyString(stmt.Module, false))
 			}
 			m.requireSemicolon()
 		}
 	case *js.DirectivePrologueStmt:
+		stmt.Value[0] = '"'
+		stmt.Value[len(stmt.Value)-1] = '"'
 		m.write(stmt.Value)
 		m.requireSemicolon()
 	}
@@ -420,7 +426,7 @@ func (m *jsMinifier) minifyBlockAsStmt(blockStmt *js.BlockStmt) {
 	// minify block when statement is expected, i.e. semicolon if empty or remove braces for single statement
 	// assume we already renamed the scope
 	hasLexicalVars := false
-	for _, v := range blockStmt.Scope.Declared[blockStmt.Scope.NumForDeclared:] {
+	for _, v := range blockStmt.Scope.Declared[blockStmt.Scope.NumForDecls:] {
 		if v.Decl == js.LexicalDecl {
 			hasLexicalVars = true
 			break
@@ -459,7 +465,7 @@ func (m *jsMinifier) minifyStmtOrBlock(i js.IStmt, blockType blockType) {
 func (m *jsMinifier) minifyAlias(alias js.Alias) {
 	if alias.Name != nil {
 		if alias.Name[0] == '"' || alias.Name[0] == '\'' {
-			m.write(minifyString(alias.Name))
+			m.write(minifyString(alias.Name, false))
 		} else {
 			m.write(alias.Name)
 		}
@@ -470,7 +476,7 @@ func (m *jsMinifier) minifyAlias(alias js.Alias) {
 	}
 	if alias.Binding != nil {
 		if alias.Binding[0] == '"' || alias.Binding[0] == '\'' {
-			m.write(minifyString(alias.Binding))
+			m.write(minifyString(alias.Binding, false))
 		} else {
 			m.write(alias.Binding)
 		}
@@ -732,19 +738,26 @@ func (m *jsMinifier) minifyClassDecl(decl *js.ClassDecl) {
 	}
 	m.write(openBraceBytes)
 	m.needsSemicolon = false
-	for _, item := range decl.Definitions {
+	for _, item := range decl.List {
 		m.writeSemicolon()
-		m.minifyPropertyName(item.Name)
-		if item.Init != nil {
-			m.write(equalBytes)
-			m.minifyExpr(item.Init, js.OpAssign)
-		}
-		m.requireSemicolon()
-	}
-	if 0 < len(decl.Methods) {
-		m.writeSemicolon()
-		for _, item := range decl.Methods {
-			m.minifyMethodDecl(item)
+		if item.StaticBlock != nil {
+			m.write(staticBytes)
+			m.minifyBlockStmt(item.StaticBlock)
+		} else if item.Method != nil {
+			m.minifyMethodDecl(item.Method)
+		} else {
+			if item.Static {
+				m.write(staticBytes)
+				if !item.Name.IsComputed() && item.Name.Literal.TokenType == js.IdentifierToken {
+					m.write(spaceBytes)
+				}
+			}
+			m.minifyPropertyName(item.Name)
+			if item.Init != nil {
+				m.write(equalBytes)
+				m.minifyExpr(item.Init, js.OpAssign)
+			}
+			m.requireSemicolon()
 		}
 	}
 	m.write(closeBraceBytes)
@@ -756,6 +769,8 @@ func (m *jsMinifier) minifyPropertyName(name js.PropertyName) {
 		m.write(openBracketBytes)
 		m.minifyExpr(name.Computed, js.OpAssign)
 		m.write(closeBracketBytes)
+	} else if name.Literal.TokenType == js.StringToken {
+		m.write(minifyString(name.Literal.Data, false))
 	} else {
 		m.write(name.Literal.Data)
 	}
@@ -838,45 +853,6 @@ func (m *jsMinifier) minifyBinding(ibinding js.IBinding) {
 	}
 }
 
-func (m *jsMinifier) minifyBinaryExpr(expr *js.BinaryExpr) bool {
-	if lit, ok := expr.Y.(*js.LiteralExpr); ok && lit.TokenType == js.StringToken && expr.Op == js.AddToken {
-		// merge strings that are added together
-		n := len(lit.Data) - 2
-		strings := []*js.LiteralExpr{lit}
-
-		left := expr
-		for {
-			if 50 < len(strings) {
-				return false // limit recursion
-			} else if lit, ok := left.X.(*js.LiteralExpr); ok && lit.TokenType == js.StringToken {
-				n += len(lit.Data) - 2
-				strings = append(strings, lit)
-				break
-			} else if left, ok = left.X.(*js.BinaryExpr); ok && left.Op == js.AddToken {
-				if lit, ok := left.Y.(*js.LiteralExpr); ok && lit.TokenType == js.StringToken {
-					n += len(lit.Data) - 2
-					strings = append(strings, lit)
-					continue
-				}
-			}
-			return false
-		}
-
-		b := make([]byte, 0, n+2)
-		b = append(b, strings[len(strings)-1].Data[:len(strings[len(strings)-1].Data)-1]...)
-		for i := len(strings) - 2; 0 < i; i-- {
-			b = append(b, strings[i].Data[1:len(strings[i].Data)-1]...)
-		}
-		b = append(b, strings[0].Data[1:]...)
-		b[len(b)-1] = b[0]
-
-		// unescaped quotes will be repaired in minifyString
-		m.write(minifyString(b))
-		return true
-	}
-	return false
-}
-
 func (m *jsMinifier) minifyExpr(i js.IExpr, prec js.OpPrec) {
 	if cond, ok := i.(*js.CondExpr); ok {
 		i = m.optimizeCondExpr(cond, prec)
@@ -927,7 +903,7 @@ func (m *jsMinifier) minifyExpr(i js.IExpr, prec js.OpPrec) {
 				m.write(notOneBytes)
 			}
 		} else if expr.TokenType == js.StringToken {
-			m.write(minifyString(expr.Data))
+			m.write(minifyString(expr.Data, true))
 		} else if expr.TokenType == js.RegExpToken {
 			// </script>/ => < /script>/
 			if 0 < len(m.prev) && m.prev[len(m.prev)-1] == '<' && bytes.HasPrefix(expr.Data, regExpScriptBytes) {
@@ -938,7 +914,9 @@ func (m *jsMinifier) minifyExpr(i js.IExpr, prec js.OpPrec) {
 			m.write(expr.Data)
 		}
 	case *js.BinaryExpr:
-		if m.minifyBinaryExpr(expr) {
+		mergeBinaryExpr(expr)
+		if expr.X == nil {
+			m.minifyExpr(expr.Y, prec)
 			break
 		}
 
@@ -974,9 +952,18 @@ func (m *jsMinifier) minifyExpr(i js.IExpr, prec js.OpPrec) {
 			//	}
 			//}
 
+			if v, not, ok := isUndefinedOrNullVar(expr); ok {
+				// change a===null||a===undefined to a==null
+				op := js.EqEqToken
+				if not {
+					op = js.NotEqToken
+				}
+				expr = &js.BinaryExpr{op, v, &js.LiteralExpr{js.NullToken, nullBytes}}
+			}
+
 			m.minifyExpr(expr.X, precLeft)
-			// 0 < len(m.prev) always
 			if expr.Op == js.GtToken && m.prev[len(m.prev)-1] == '-' {
+				// 0 < len(m.prev) always
 				m.write(spaceBytes)
 			} else if expr.Op == js.EqEqEqToken || expr.Op == js.NotEqEqToken {
 				if left, ok := expr.X.(*js.UnaryExpr); ok && left.Op == js.TypeofToken {
@@ -1070,8 +1057,10 @@ func (m *jsMinifier) minifyExpr(i js.IExpr, prec js.OpPrec) {
 		} else {
 			m.minifyExpr(expr.X, js.OpMember)
 		}
-		// 0 < len(m.prev) always
-		if last := m.prev[len(m.prev)-1]; '0' <= last && last <= '9' {
+		if expr.Optional {
+			m.write(questionBytes)
+		} else if last := m.prev[len(m.prev)-1]; '0' <= last && last <= '9' {
+			// 0 < len(m.prev) always
 			isInteger := true
 			for _, c := range m.prev[:len(m.prev)-1] {
 				if c < '0' || '9' < c {
@@ -1146,6 +1135,9 @@ func (m *jsMinifier) minifyExpr(i js.IExpr, prec js.OpPrec) {
 			} else {
 				m.minifyExpr(expr.Tag, js.OpMember)
 			}
+			if expr.Optional {
+				m.write(optChainBytes)
+			}
 		}
 		parentInFor := m.inFor
 		m.inFor = false
@@ -1196,6 +1188,9 @@ func (m *jsMinifier) minifyExpr(i js.IExpr, prec js.OpPrec) {
 		m.minifyExpr(expr.X, js.OpCall)
 		parentInFor := m.inFor
 		m.inFor = false
+		if expr.Optional {
+			m.write(optChainBytes)
+		}
 		m.minifyArguments(expr.Args)
 		m.inFor = parentInFor
 	case *js.IndexExpr:
@@ -1209,6 +1204,9 @@ func (m *jsMinifier) minifyExpr(i js.IExpr, prec js.OpPrec) {
 		} else {
 			m.minifyExpr(expr.X, js.OpMember)
 		}
+		if expr.Optional {
+			m.write(optChainBytes)
+		}
 		if lit, ok := expr.Y.(*js.LiteralExpr); ok && lit.TokenType == js.StringToken && 2 < len(lit.Data) {
 			if isIdent := js.AsIdentifierName(lit.Data[1 : len(lit.Data)-1]); isIdent {
 				m.write(dotBytes)
@@ -1233,18 +1231,6 @@ func (m *jsMinifier) minifyExpr(i js.IExpr, prec js.OpPrec) {
 		m.minifyExpr(expr.X, js.OpAssign)
 		m.write(colonBytes)
 		m.minifyExpr(expr.Y, js.OpAssign)
-	case *js.OptChainExpr:
-		m.minifyExpr(expr.X, js.OpCall)
-		m.write(optChainBytes)
-		if callExpr, ok := expr.Y.(*js.CallExpr); ok {
-			m.minifyArguments(callExpr.Args)
-		} else if indexExpr, ok := expr.Y.(*js.IndexExpr); ok {
-			m.write(openBracketBytes)
-			m.minifyExpr(indexExpr.Y, js.OpExpr)
-			m.write(closeBracketBytes)
-		} else {
-			m.minifyExpr(expr.Y, js.OpPrimary) // TemplateExpr or LiteralExpr
-		}
 	case *js.VarDecl:
 		m.minifyVarDecl(expr, true) // happens in for statement or when vars were hoisted
 	case *js.FuncDecl:
