@@ -7,6 +7,9 @@ import (
 	"github.com/tdewolff/parse/v2/js"
 )
 
+const identStartLen = 54
+const identContinueLen = 64
+
 type renamer struct {
 	identStart    []byte
 	identContinue []byte
@@ -23,10 +26,13 @@ func newRenamer(rename, useCharFreq bool) *renamer {
 	identStart := []byte("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_$")
 	identContinue := []byte("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_$0123456789")
 	if useCharFreq {
-		// sorted based on character frequency of a collection of JS samples (incl. the var names!)
+		// sorted based on character frequency of a collection of JS samples
 		identStart = []byte("etnsoiarclduhmfpgvbjy_wOxCEkASMFTzDNLRPHIBV$WUKqYGXQZJ")
 		identContinue = []byte("etnsoiarcldu14023hm8f6pg57v9bjy_wOxCEkASMFTzDNLRPHIBV$WUKqYGXQZJ")
 	}
+	if len(identStart) != identStartLen || len(identContinue) != identContinueLen {
+		panic("bad identStart or identContinue lengths")
+	}
 	identOrder := map[byte]int{}
 	for i, c := range identStart {
 		identOrder[c] = i
@@ -46,7 +52,8 @@ func (r *renamer) renameScope(scope js.Scope) {
 	}
 
 	i := 0
-	sort.Sort(js.VarsByUses(scope.Declared))
+	// keep function argument declaration order to improve GZIP compression
+	sort.Sort(js.VarsByUses(scope.Declared[scope.NumFuncArgs:]))
 	for _, v := range scope.Declared {
 		v.Data = r.getName(v.Data, i)
 		i++
@@ -77,15 +84,16 @@ func (r *renamer) isReserved(name []byte, undeclared js.VarArray) bool {
 func (r *renamer) getIndex(name []byte) int {
 	index := 0
 NameLoop:
-	for i, b := range name {
+	for i := len(name) - 1; 0 <= i; i-- {
 		chars := r.identContinue
 		if i == 0 {
 			chars = r.identStart
+			index *= identStartLen
 		} else {
-			index *= len(r.identContinue)
+			index *= identContinueLen
 		}
 		for j, c := range chars {
-			if b == c {
+			if name[i] == c {
 				index += j
 				continue NameLoop
 			}
@@ -93,9 +101,9 @@ NameLoop:
 		return -1
 	}
 	for n := 0; n < len(name)-1; n++ {
-		offset := len(r.identStart)
+		offset := identStartLen
 		for i := 0; i < n; i++ {
-			offset *= len(r.identContinue)
+			offset *= identContinueLen
 		}
 		index += offset
 	}
@@ -107,17 +115,19 @@ func (r *renamer) getName(name []byte, index int) []byte {
 	// Thus we can have 54 one-character names and 52*54=2808 two-character names for every branch leaf.
 	// That is sufficient for virtually all input.
 
-	if index < len(r.identStart) {
+	// one character
+	if index < identStartLen {
 		name[0] = r.identStart[index]
 		return name[:1]
 	}
+	index -= identStartLen
 
-	index -= len(r.identStart)
+	// two characters or more
 	n := 2
 	for {
-		offset := len(r.identStart)
+		offset := identStartLen
 		for i := 0; i < n-1; i++ {
-			offset *= len(r.identContinue)
+			offset *= identContinueLen
 		}
 		if index < offset {
 			break
@@ -131,11 +141,12 @@ func (r *renamer) getName(name []byte, index int) []byte {
 	} else {
 		name = name[:n]
 	}
-	for j := n - 1; 0 < j; j-- {
-		name[j] = r.identContinue[index%len(r.identContinue)]
-		index /= len(r.identContinue)
+	name[0] = r.identStart[index%identStartLen]
+	index /= identStartLen
+	for i := 1; i < n; i++ {
+		name[i] = r.identContinue[index%identContinueLen]
+		index /= identContinueLen
 	}
-	name[0] = r.identStart[index]
 	return name
 }
 
